open ClientServer
type 'a t = {
  started : bool;
  eb : 'a message EventBroker.t;
  socket : Unix.file_descr;
  id : int;
  server : int
}
let rec thread_func a =
  let r = match EventBroker.poll a.eb a.id with
  | Some m -> (
    match m with
    | Quit ->
      EventBroker.send a.eb a.server (Join (Thread.self ()));
      true
    | _ -> false
  )
  | None -> false
  in
  if r = true
  then ()
  else (
    match Unix.select [a.socket] [] [] 0.001 with
    | [], _, _ -> thread_func a
    | h::_, _, _ ->
      let f,_ = Unix.accept a.socket in
      EventBroker.send a.eb a.server (Connect(f));
      thread_func a
  )
let create eb s fd =
  let id = EventBroker.channel eb in
  {
    started = false;
    eb = eb;
    socket = fd;
    id = id;
    server = s
  }
let id a = a.id
let start a = match a.started with
| true -> raise (Invalid_state "Acceptor already active")
| false ->
  let _ = Thread.create thread_func a in
  { a with started = true }
let stop a = match a.started with
| false -> raise (Invalid_state "Acceptor not active")
| true ->
  EventBroker.send a.eb a.id Quit;
  { a with started = false }

