(**
 * Handling functions for the acceptor thread.  This module is not really for
 * public consumption and should probably me moved inside the {!Server} module
 * and hidden away.
 *
 * !NOTE! There is a bug somewhere in the internal [thread_func] function. 
 * Under high loads, one of three things happen:
 * - Nothing (the code works as it should)
 * - [Unix.accept] fails with an error 12 (at a random time)
 * - Segmentation Fault (at a random time)
 *
 * I'm not sure where the bug is: my tester, the [Acceptor] module, OCaml, or
 * Linux...  Hmmm....
 * 
 * @author Jonathan Bryant
 *)

type 'a t
(** The abstract type of the acceptor thread *)

val create : 'a ClientServer.message EventBroker.t -> int -> Unix.file_descr -> 'a t
(** [Acceptor.create eb q fd] creates a new acceptor using the {!EventBroker}
  * [eb], the queue number [q], and accepting connections from the UNIX file
  * descriptor [fd]. *)

val id : 'a t -> int
(** [Acceptor.id a] gets the identification number of the acceptor [a]. *)

val start : 'a t -> 'a t
(** [Acceptor.start a] starts the acceptor [a] running on its own thread.
 *  Raises {!ClientServer.Invalid_state} if the acceptor is already running. *)

val stop : 'a t -> 'a t
(** [Acceptor.stop a] stops the running acceptor [a].  Raises
 *  {!ClientServer.Invalid_state} if the acceptor is not running *)
