#!/bin/bash

# Compiler
COMPILER="ocamlopt.opt"

# Preclean and setup
echo "+-Precleaning"
rm -rf doc/
rm -f *.cmi *.cmx *.cmxa *.o *.a
mkdir doc

# Compiler flags
FLAGS="-thread unix.cmxa threads.cmxa"

# Library files
ML_FILES="clientServer eventBroker connection acceptor server client"
LIB_FILES=""
DOC_FILES=""

# Compile the library
echo "+-Compiling the library"
for FILE in $ML_FILES; do
  echo "  +-Compiling interface $FILE.mli"
  $COMPILER -c $FLAGS $FILE.mli
  echo "  +-Compiling implementation $FILE.ml"
  $COMPILER -c $FLAGS $FILE.ml
  LIB_FILES="$LIB_FILES $FILE.cmx"
  DOC_FILES="$DOC_FILES $FILE.mli $FILE.ml"
done

# Build the library
echo "+-Linking the library"
$COMPILER -a -o clientServer.cmxa -thread $LIB_FILES

# Build the tester
echo "+-Building the tester"
$COMPILER -o clientServerTest $FLAGS clientServer.cmxa clientServerTest.ml

# Build the documentation
echo "+-Building the documentation"
ocamldoc.opt -d doc -I +threads -html -stars -keep-code -colorize-code $DOC_FILES
