open ClientServer
type 'a t = {
  connected : bool;
  socket : Unix.file_descr;
  ic : in_channel;
  oc : out_channel;
  serialize : (out_channel -> 'a -> unit);
  deserialize : (in_channel -> 'a);
  handle : ('a -> 'a option) option;
  mutable thread : Thread.t;
  mutable stop : bool
}
let poll c = match Unix.select [c.socket] [] [] 0.001 with
| [], _, _ -> None
| h::_, _, _ -> Some (c.deserialize c.ic)
let rec thread_func c =
  let hndlr = match c.handle with
  | Some f -> f
  | _ -> failwith "Should be no thread if no handler function... Oops..."
  in
  match c.stop, poll c with
  | false, Some m -> (
    match hndlr m with
    | Some m -> c.serialize c.oc m; thread_func c
    | None -> thread_func c
  )
  | false, None -> Thread.yield (); thread_func c
  | true, _ -> ()
let create fd = {
  connected = false;
  socket = fd;
  ic = Unix.in_channel_of_descr fd;
  oc = Unix.out_channel_of_descr fd;
  serialize = (fun oc -> fun m -> Marshal.to_channel oc m []; flush oc);
  deserialize = (fun ic -> (Marshal.from_channel ic : 'a));
  handle = None;
  thread = Thread.self ();
  stop = false
}
let connect c = match c.connected, c.handle with
  | false, Some m ->
    let t = Thread.create thread_func c in
    c.thread <- t;
    { c with connected = true }
  | false, None -> { c with connected = true }
  | true, _ -> raise (Invalid_state "Client already active")
let send c m = c.serialize c.oc m
let rec receive c = match poll c with
| Some m -> m
| None -> Thread.yield (); receive c
let serialize f c = match c.connected, f with
| false, Some f -> { c with serialize = f }
| false, None -> { c with serialize = (fun oc -> fun m ->
    Marshal.to_channel oc m [];
    flush oc
)}
| true, _ -> raise (Invalid_state "Client already active")
let deserialize f c = match c.connected, f with
| false, Some f -> { c with deserialize = f }
| false, None -> { c with deserialize = (fun ic ->
    (Marshal.from_channel ic : 'a)
)}
| true, _ -> raise (Invalid_state "Client already active")
let handler f c = match c.connected with
| false -> { c with handle = f }
| true -> raise (Invalid_state "Client already active")
let serializer f c = match c.connected, f with
| false, Some f -> { c with serialize = f }
| false, None -> { c with serialize = (fun oc -> fun m ->
    Marshal.to_channel oc m [];
    flush oc
)}
| true, _ -> raise (Invalid_state "Client already active")
let deserializer f c = match c.connected, f with
| false, Some f -> { c with deserialize = f }
| false, None -> { c with deserialize = (fun ic ->
    (Marshal.from_channel ic : 'a)
)}
| true, _ -> raise (Invalid_state "Client already active")
let disconnect c = match c.connected, c.handle with
| true, Some _ ->
  c.stop <- true;
  Thread.join c.thread;
  { c with connected = false }
| true, None -> { c with connected = false }
| false, _ -> raise (Invalid_state "Client not active")

