type msg =
| Syn
| Ack
let () =
  Random.self_init ();

  (* Create, setup, and start the server *)
  let server_socket = Unix.socket Unix.PF_UNIX Unix.SOCK_STREAM 0 in
  Unix.bind server_socket (Unix.ADDR_UNIX("/tmp/test_sock"));
  Unix.listen server_socket 10000;
  let event_broker = EventBroker.create () in
  let server = Server.create event_broker server_socket in
  let server = Server.connect (Some (fun c ->
    Printf.printf "Connection Accepted (ID: %d)\n" (Connection.id c);
    c
  )) server in
  let server = Server.disconnect (Some (fun c ->
    Printf.printf "Connection Lost (ID: %d)\n" (Connection.id c);
    c
  )) server in
  let server = Server.handler (Some (fun m -> match m with
  | Syn -> Some Ack
  | _ -> Printf.printf "Ignoring unknown message\n"; None
  )) server in
  let server = Server.start server in

  (* Some closures to do the pinging: up to 10 threads each pinging up to
   * 1,000,000 times *)
  let rec ping client count =
    if count = 0 then () else (
      Client.send client Syn;
      let _ = Client.receive client in
      ping client (count - 1)
    )
  in
  let rec loop count acc =
    if count = 0 then acc else (
      let s = Unix.socket Unix.PF_UNIX Unix.SOCK_STREAM 0 in
      Unix.connect s (Unix.ADDR_UNIX("/tmp/test_sock"));
      let c = Client.create s in
      let t = Thread.create (fun x ->
        ping c x; Unix.shutdown s Unix.SHUTDOWN_ALL
      ) ((Random.int 1000000) + 1) in
      loop (count - 1) (t::acc)
    )
  in

  (* Run the stress test *)
  List.iter (fun t -> Thread.join t) (loop ((Random.int 10) + 1) []);

  (* Shutdown the server an clean up *)
  let server = Server.stop server in
  Unix.shutdown server_socket Unix.SHUTDOWN_ALL;
  Unix.unlink "/tmp/test_sock"
