open ClientServer
type 'a t = {
  started : bool;
  eb : 'a message EventBroker.t;
  id : int;
  server : int;
  socket : Unix.file_descr;
  ic : in_channel;
  oc : out_channel;
  serialize : out_channel -> 'a -> unit;
  deserialize : in_channel -> 'a;
  handle : 'a -> 'a option
}
let id c = c.id
let rec thread_func c =
  let r = match EventBroker.poll c.eb c.id with
  | Some m -> (
    match m with
    | Quit ->
      EventBroker.send c.eb c.server (Disconnect (id c));
      EventBroker.send c.eb c.server (Join (Thread.self ()));
      true
    | Send msg ->
      c.serialize c.oc msg;
      false
    | _ -> false
  )
  | None -> false
  in
  if r = true
  then ()
  else (
    try
      match Unix.select [c.socket] [] [] 0.001 with
      | [], _, _ -> thread_func c
      | h::_, _, _ -> (
        match c.handle (c.deserialize c.ic) with
        | Some m -> c.serialize c.oc m; thread_func c
        | None -> thread_func c
      )
    with End_of_file ->
      EventBroker.send c.eb c.id (Quit);
      thread_func c
  )
let create eb s fd =
  let id = EventBroker.channel eb in
  {
    started = false;
    eb = eb;
    id = id;
    server = s;
    socket = fd;
    ic = Unix.in_channel_of_descr fd;
    oc = Unix.out_channel_of_descr fd;
    serialize = (fun oc -> fun m -> Marshal.to_channel oc m []; flush oc);
    deserialize = (fun ic -> (Marshal.from_channel ic : 'a));
    handle = (fun m -> None)
  }
let deserializer f c = match c.started, f with
| false, Some f -> { c with deserialize = f }
| false, None -> { c with deserialize = (fun ic ->
    (Marshal.from_channel ic : 'a)
)}
| true, _ -> raise (Invalid_state "Connection already active")
let handler f c = match c.started, f with
| false, Some f -> { c with handle = f }
| false, None -> { c with handle = (fun m -> None) }
| true, _ -> raise (Invalid_state "Connection already active")
let serializer f c = match c.started, f with
| false, Some f -> { c with serialize = f }
| false, None -> { c with serialize = (fun oc -> fun m ->
    Marshal.to_channel oc m [];
    flush oc
)}
| true, _ -> raise (Invalid_state "Connection already active")
let start c = match c.started with
| true -> raise (Invalid_state "Conenction already active")
| false ->
  let t = Thread.create thread_func c in
  { c with started = true }
let send c m = match c.started with
| false -> raise (Invalid_state "Connection not active")
| true -> EventBroker.send c.eb c.id (Send m)
let stop c = match c.started with
| false -> raise (Invalid_state "Connection not active")
| true ->
  EventBroker.send c.eb c.id Quit;
  { c with started = false }

