open ClientServer
type 'a t = { 
  mutex : Mutex.t;
  mutable count : int;
  ct : (int, 'a Queue.t) Hashtbl.t;
  mutable free : int list
}
let create () = {
  mutex = Mutex.create ();
  count = 0;
  ct = Hashtbl.create 27;
  free = []
}
let channel eb =
  Mutex.lock eb.mutex;
  let id = match eb.free with
  | [] ->
    Hashtbl.replace eb.ct (eb.count + 1) (Queue.create ());
    eb.count <- eb.count + 1;
    eb.count
  | h::t ->
    Hashtbl.replace eb.ct h (Queue.create ());
    eb.free <- t;
    h
  in
  Mutex.unlock eb.mutex;
  id
let remove eb c =
  Mutex.lock eb.mutex;
  Hashtbl.remove eb.ct c;
  eb.count <- eb.count - 1;
  eb.free <- (c::eb.free);
  Mutex.unlock eb.mutex
let send eb c m =
  Mutex.lock eb.mutex;
  Queue.add m (Hashtbl.find eb.ct c);
  Mutex.unlock eb.mutex
let poll eb c =
  try
    Mutex.lock eb.mutex;
    let m = Queue.take (Hashtbl.find eb.ct c) in
    Mutex.unlock eb.mutex;
    (Some m)
  with Queue.Empty ->
    Mutex.unlock eb.mutex;
    None
let rec receive eb c = match poll eb c with
| Some m -> m
| None -> Thread.yield (); receive eb c

