(**
 * Asynchronous buffered message passing between threads.
 * @author Jonathan Bryant
 *)

type 'a t
(** The abstract type of Event Brokers *)

val create : unit -> 'a t
(** [EventBroker.create ()] creates a new (thread safe, obviously) event broker.
 * *)

val channel : 'a t -> int
(** [EventBroker.channel eb] creates a new channel in the event broker [eb] and
 *  the channel identification number is returned. *)

val remove : 'a t -> int -> unit
(** [EventBroker.remove eb c] removes the channel [c] from the event broker [b].
 * *)

val send : 'a t -> int -> 'a -> unit
(** [EventBroker.send eb c m] sends the message [m] to the channel [c] on the
 *  event broker [eb]. *)

val poll : 'a t -> int -> 'a option
(** [EventBroker.poll eb c] retreives a message from the channel [c] on the
 *  event broker [eb].  If there is no pending message, then this function
 *  returns [None]. *)

val receive : 'a t -> int -> 'a
(** [EventBroker.reveive eb c] retreives a message from the channel [c] on the
 *  event broker [eb]. *)
